%% @file 
% est_all_lag.m
%
% @author Lars Johannesen
%
% @section DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%
% @section Examples:
%  [lags, aux] = est_all_lag(out, wavf, min_sqi, max_lag)
%
% @section See also:
%

%% 
% @brief Determines the lag for all signals
% @param out From combined
% @param wavf Waveform for timeseries
% @param min_sqi minimum sqi
% @param max_lag maximum lag with xcorr
% @return [lags, aux] = vector and cell-array for each comparison
%
function [lags,aux] = est_all_lag(out, wavf, min_sqi,max_lag)
%est_all_lag Estimates lag for all signals
%
% est_all_lag(out, wavf, min_sqi, max_lag)c
%
% out From combined
% wavf Waveform for timeseries
% min_sqi minimum sqi
% max_lag maximum lag with xcorr
%
%                                  DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%
% Written by: Lars Johannesen
%
% Examples:
%  [lags, aux] = est_all_lag(out, wavf, 1, 3000)
%
% See also:
	lags = zeros(1,length(out));
	len = out{1}.length;
	aux = cell(1,length(out));

	for i=2:length(out)
		[l,a] = est_lag(out, 1, i, wavf, min_sqi, max_lag);
		lags(i) = l;
		aux{i} = l;
	end	
end
