/********************************************************************************
 *                                  DISCLAIMER                                  *
 * This software and documentation (the "Software") were developed at the Food  *
 * and Drug Administration (FDA) by employees of the Federal Government in the  *
 * course of their official duties. Pursuant to Title 17, Section 105 of the    *
 * United States Code, this work is not subject to copyright protection and is  *
 * in the public domain. Permission is hereby granted, free of charge, to any   *
 * person obtaining a copy of the Software, to deal in the Software without     *
 * restriction, including without limitation the rights to use, copy, modify,   *
 * merge, publish, distribute, sublicense, or sell copies of the Software or    *
 * derivatives, and to permit persons to whom the Software is furnished to do   *
 * so. FDA assumes no responsibility whatsoever for use by other parties of     *
 * the Software, its source code, documentation or compiled executables, and    *
 * makes no guarantees, expressed or implied, about its quality, reliability,   *
 *  or any other characteristic. Further, use of this code in no way implies    *
 * endorsement by the FDA or confers any advantage in regulatory decisions.     *
 * Although this software can be redistributed and/or modified freely, we       *
 * ask that any derivative works bear some notice that they are derived from    *
 * it, and any modified versions bear some notice that they have been modified. * 
 *                                                                              *
 * Written by:                                                                  *
 *  Lars Johannesen                                                             *
 ********************************************************************************/
#include "mex.h"

#include <wfdb/wfdb.h>
#include <wfdb/ecgcodes.h>

#include <vector>
#include <map>
#include <string>

#include <iostream>

#include <stdexcept>

#include <cmath>

#include <armadillo>

template<typename TYP>
TYP get_matlab_scalar(const mxArray *a) {
	TYP *t = (TYP*)mxGetData(a);
	return *t;
}

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
	if(nrhs != 4) {
		mexErrMsgTxt("Need one arguments: record fs ann anns");
	}

	try {
		if(!mxIsChar(prhs[0]))
                        mexErrMsgTxt("First argument must be a char");

		if(!mxIsDouble(prhs[1]))
                        mexErrMsgTxt("Second argument must be a double");

		if(!mxIsChar(prhs[2]))
                        mexErrMsgTxt("Third argument must be a char");

		if(!mxIsDouble(prhs[3]))
                        mexErrMsgTxt("Fourth argument must be a double");

		char *rec = mxArrayToString(prhs[0]);
		char *ann = mxArrayToString(prhs[2]);

		int cols = mxGetN(prhs[3]);
		int rows = mxGetM(prhs[3]);

		double fs = get_matlab_scalar<double>(prhs[1]);

		if(cols != 1 && rows != 1) {
			mexErrMsgTxt("Should be vector!");
		}

		int nelem = cols == 1 ? rows : cols;

		const double *annvec = (const double*) mxGetPr(prhs[3]);
		arma::vec saveanns = arma::zeros<arma::vec>(nelem);
		std::size_t cnt = 0;
		for(std::size_t i = 0; i < nelem; ++i) {
			if(annvec[i] >= 0) {
				saveanns(cnt) = annvec[i];
				++cnt;
			}
		}

		if (cnt == 0) {
			mexErrMsgTxt("No annotations >= 0");
		}

		saveanns = saveanns.subvec(0,cnt-1);

		WFDB_Anninfo a;
		a.name = ann;
		a.stat = WFDB_WRITE;

		WFDB_Annotation annot;
		annot.anntyp = NORMAL;
		annot.aux = NULL;
		annot.subtyp = annot.chan = annot.num = 0;

		if(annopen(rec, &a, 1) < 0) {
			mexErrMsgTxt("Could not open ann!");
		}

		for(std::size_t i = 0; i < saveanns.n_elem; ++i) {
			int loc = static_cast<int>(round(fs/1000*saveanns(i)));
			annot.time = loc;
			putann(0, &annot);
		}

		wfdbquit();
	} catch (...) {
		mexErrMsgTxt("Failed loading");
	}
}
